@@ -11,6 +11,8 @@ from inspect import isclass
 from inspect import isfunction
 import sys
 
+import six
+
 from ddtrace import Pin
 from ddtrace import config
 from ddtrace.constants import ANALYTICS_SAMPLE_RATE_KEY
@@ -25,7 +27,6 @@ from ddtrace.propagation.utils import from_wsgi_header
 from ddtrace.utils.formats import asbool
 from ddtrace.utils.formats import get_env
 from ddtrace.vendor import debtcollector
-from ddtrace.vendor import six
 from ddtrace.vendor import wrapt
 
 from . import conf
@@ -44,6 +45,7 @@ config._add(
         cache_service_name=get_env("django", "cache_service_name") or "django",
         database_service_name_prefix=get_env("django", "database_service_name_prefix", default=""),
         database_service_name=get_env("django", "database_service_name", default=""),
+        trace_fetch_methods=asbool(get_env("django", "trace_fetch_methods", default=False)),
         distributed_tracing_enabled=True,
         instrument_middleware=asbool(get_env("django", "instrument_middleware", default=True)),
         instrument_databases=True,
@@ -58,6 +60,11 @@ config._add(
 )
 
 
+# Set on patch, when django is imported
+Resolver404 = None
+DJANGO22 = None
+
+
 def patch_conn(django, conn):
     def cursor(django, pin, func, instance, args, kwargs):
         alias = getattr(conn, "alias", "default")
@@ -317,52 +324,59 @@ def traced_get_response(django, pin, func, instance, args, kwargs):
     if request is None:
         return func(*args, **kwargs)
 
-    try:
-        request_headers = request.META
+    trace_utils.activate_distributed_headers(pin.tracer, int_config=config.django, request_headers=request.META)
 
-        trace_utils.activate_distributed_headers(pin.tracer, int_config=config.django, request_headers=request_headers)
+    with pin.tracer.trace(
+        "django.request",
+        resource=request.method,
+        service=trace_utils.int_service(pin, config.django),
+        span_type=SpanTypes.WEB,
+    ) as span:
+        span.metrics[SPAN_MEASURED_KEY] = 1
 
-        # Determine the resolver and resource name for this request
-        resolver = get_resolver(getattr(request, "urlconf", None))
+        analytics_sr = config.django.get_analytics_sample_rate(use_global_config=True)
+        if analytics_sr is not None:
+            span.set_tag(ANALYTICS_SAMPLE_RATE_KEY, analytics_sr)
 
-        if django.VERSION < (1, 10, 0):
-            error_type_404 = django.core.urlresolvers.Resolver404
-        else:
-            error_type_404 = django.urls.exceptions.Resolver404
+        # Set HTTP Request tags
+        response = func(*args, **kwargs)
 
-        route = None
-        resolver_match = None
-        resource = request.method
         try:
-            # Resolve the requested url and build resource name pieces
-            resolver_match = resolver.resolve(request.path_info)
-            handler, _, _ = resolver_match
-            handler = func_name(handler)
-            urlpattern = ""
-            resource_format = None
+            # Get resolver match result and build resource name pieces
+            resolver_match = request.resolver_match
+            if not resolver_match:
+                # The request quite likely failed (e.g. 404) so we do the resolution anyway.
+                resolver = get_resolver(getattr(request, "urlconf", None))
+                resolver_match = resolver.resolve(request.path_info)
+            handler = func_name(resolver_match[0])
 
             if config.django.use_handler_resource_format:
-                resource_format = "{method} {handler}"
+                span.resource = " ".join((span.resource, handler))
             elif config.django.use_legacy_resource_format:
-                resource_format = "{handler}"
+                span.resource = handler
             else:
                 # In Django >= 2.2.0 we can access the original route or regex pattern
                 # TODO: Validate if `resolver.pattern.regex.pattern` is available on django<2.2
-                if django.VERSION >= (2, 2, 0):
-                    route = utils.get_django_2_route(resolver, resolver_match)
-                    resource_format = "{method} {urlpattern}"
+                if DJANGO22:
+                    # Determine the resolver and resource name for this request
+                    route = utils.get_django_2_route(request, resolver_match)
+                    if route:
+                        span.resource = " ".join((request.method, route))
+                        span._set_str_tag("http.route", route)
                 else:
-                    resource_format = "{method} {handler}"
+                    span.resource = " ".join((request.method, handler))
 
-                if route is not None:
-                    urlpattern = route
+            span._set_str_tag("django.view", resolver_match.view_name)
+            utils.set_tag_array(span, "django.namespace", resolver_match.namespaces)
 
-            resource = resource_format.format(method=request.method, urlpattern=urlpattern, handler=handler)
+            # Django >= 2.0.0
+            if hasattr(resolver_match, "app_names"):
+                utils.set_tag_array(span, "django.app", resolver_match.app_names)
 
-        except error_type_404:
+        except Resolver404:
             # Normalize all 404 requests into a single resource name
             # DEV: This is for potential cardinality issues
-            resource = "{0} 404".format(request.method)
+            span.resource = " ".join((request.method, "404"))
         except Exception:
             log.debug(
                 "Failed to resolve request path %r with path info %r",
@@ -370,95 +384,66 @@ def traced_get_response(django, pin, func, instance, args, kwargs):
                 getattr(request, "path_info", "not-set"),
                 exc_info=True,
             )
-    except Exception:
-        log.debug("Failed to trace django request %r", args, exc_info=True)
-        return func(*args, **kwargs)
-    else:
-        with pin.tracer.trace(
-            "django.request",
-            resource=resource,
-            service=trace_utils.int_service(pin, config.django),
-            span_type=SpanTypes.WEB,
-        ) as span:
-            span.metrics[SPAN_MEASURED_KEY] = 1
-            analytics_sr = config.django.get_analytics_sample_rate(use_global_config=True)
-            if analytics_sr is not None:
-                span.set_tag(ANALYTICS_SAMPLE_RATE_KEY, analytics_sr)
 
-            # Not a 404 request
-            if resolver_match:
-                span._set_str_tag("django.view", resolver_match.view_name)
-                utils.set_tag_array(span, "django.namespace", resolver_match.namespaces)
+        # Note: this call must be done after the function call because
+        # some attributes (like `user`) are added to the request through
+        # the middleware chain
+        _set_request_tags(django, span, request)
 
-                # Django >= 2.0.0
-                if hasattr(resolver_match, "app_names"):
-                    utils.set_tag_array(span, "django.app", resolver_match.app_names)
+        if response:
+            status = response.status_code
+            span._set_str_tag("django.response.class", func_name(response))
+            if hasattr(response, "template_name"):
+                # template_name is a bit of a misnomer, as it could be any of:
+                # a list of strings, a tuple of strings, a single string, or an instance of Template
+                # for more detail, see:
+                # https://docs.djangoproject.com/en/3.0/ref/template-response/#django.template.response.SimpleTemplateResponse.template_name
+                template = response.template_name
 
-            if route:
-                span._set_str_tag("http.route", route)
-
-            # Set HTTP Request tags
-            response = func(*args, **kwargs)
-
-            # Note: this call must be done after the function call because
-            # some attributes (like `user`) are added to the request through
-            # the middleware chain
-            _set_request_tags(django, span, request)
-
-            if response:
-                status = response.status_code
-                span._set_str_tag("django.response.class", func_name(response))
-                if hasattr(response, "template_name"):
-                    # template_name is a bit of a misnomer, as it could be any of:
-                    # a list of strings, a tuple of strings, a single string, or an instance of Template
-                    # for more detail, see:
-                    # https://docs.djangoproject.com/en/3.0/ref/template-response/#django.template.response.SimpleTemplateResponse.template_name
-                    template = response.template_name
-
-                    if isinstance(template, six.string_types):
-                        template_names = [template]
-                    elif isinstance(
-                        template,
-                        (
-                            list,
-                            tuple,
-                        ),
-                    ):
-                        template_names = template
-                    elif hasattr(template, "template"):
-                        # ^ checking by attribute here because
-                        # django backend implementations don't have a common base
-                        # `.template` is also the most consistent across django versions
-                        template_names = [template.template.name]
-                    else:
-                        template_names = None
-
-                    utils.set_tag_array(span, "django.response.template", template_names)
-
-                url = utils.get_request_uri(request)
-
-                if django.VERSION >= (2, 2, 0):
-                    request_headers = request.headers
+                if isinstance(template, six.string_types):
+                    template_names = [template]
+                elif isinstance(
+                    template,
+                    (
+                        list,
+                        tuple,
+                    ),
+                ):
+                    template_names = template
+                elif hasattr(template, "template"):
+                    # ^ checking by attribute here because
+                    # django backend implementations don't have a common base
+                    # `.template` is also the most consistent across django versions
+                    template_names = [template.template.name]
                 else:
-                    request_headers = {}
-                    for header, value in request.META.items():
-                        name = from_wsgi_header(header)
-                        if name:
-                            request_headers[name] = value
+                    template_names = None
 
-                response_headers = dict(response.items())
-                trace_utils.set_http_meta(
-                    span,
-                    config.django,
-                    method=request.method,
-                    url=url,
-                    status_code=status,
-                    query=request.META.get("QUERY_STRING", None),
-                    request_headers=request_headers,
-                    response_headers=response_headers,
-                )
+                utils.set_tag_array(span, "django.response.template", template_names)
 
-            return response
+            url = utils.get_request_uri(request)
+
+            if DJANGO22:
+                request_headers = request.headers
+            else:
+                request_headers = {}
+                for header, value in request.META.items():
+                    name = from_wsgi_header(header)
+                    if name:
+                        request_headers[name] = value
+
+            response_headers = dict(response.items())
+            trace_utils.set_http_meta(
+                span,
+                config.django,
+                method=request.method,
+                url=url,
+                status_code=status,
+                query=request.META.get("QUERY_STRING", None),
+                request_headers=request_headers,
+                response_headers=response_headers,
+            )
+
+        return response
 
 
 @trace_utils.with_traced_module
@@ -600,9 +585,18 @@ def _patch(django):
 
 
 def patch():
+    global Resolver404, DJANGO22
+
     # DEV: this import will eventually be replaced with the module given from an import hook
     import django
 
+    if django.VERSION < (1, 10, 0):
+        Resolver404 = django.core.urlresolvers.Resolver404
+    else:
+        Resolver404 = django.urls.exceptions.Resolver404
+
+    DJANGO22 = django.VERSION >= (2, 2, 0)
+
     if getattr(django, "_datadog_patch", False):
         return
     _patch(django)
